//+------------------------------------------------------------------+
//|                                                     Strategy.mqh |
//|                                 Copyright 2015, Vasiliy Sokolov. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Vasiliy Sokolov."
#property link      "http://www.mql5.com"
#property strict
#define SOURCE ("ID " + (string)ExpertMagic() + "  " + __FUNCTION__) 
#include <Object.mqh>
#include <XML\XMLBase.mqh>          //   XML
#include "Logs.mqh"                 // 
#include "PositionMT5.mqh"          //   
#include "TradeEnvironment.mqh"     //      
#include "NewBarDetector.mqh"       //   
#include "NewTickDetector.mqh"      //   
#include "Series.mqh"               //     OHLCV  
#include "TradeControl.mqh"         //         
#include "TradeState.mqh"           //         
#include "MoneyManagment.mqh"
//+------------------------------------------------------------------+
//|    .                                |
//+------------------------------------------------------------------+
enum ENUM_MARKET_EVENT_TYPE
  {
   MARKET_EVENT_TICK,               //      
   MARKET_EVENT_BAR_OPEN,           //      
   MARKET_EVENT_TIMER,              //  
   MARKET_EVENT_BOOK_EVENT          //   ( ..  ).
  };
//+------------------------------------------------------------------+
//|  ,    .                |
//+------------------------------------------------------------------+
struct MarketEvent
  {
   ENUM_MARKET_EVENT_TYPE type;     //  .
   ENUM_TIMEFRAMES   period;        //  ,     (  MARKET_EVENT_BAR_OPEN).
   string            symbol;        //  ,    .   , 
                                    // MARKET_EVENT_BOOK_EVENT,     .
  };
//+------------------------------------------------------------------+
//|       ()  |
//+------------------------------------------------------------------+
struct PositionsStat
  {
   int               open_buy;                  //      
   int               open_sell;                 //      
   int               open_total;                //     
   int               open_complex;              //   ,    
  };
//+------------------------------------------------------------------+
//|   -.                               |
//+------------------------------------------------------------------+
class CStrategy : public CObject
  {
private:
   MarketEvent       m_event;                     //     .
   ulong             m_last_changed;              //          .
   uint              m_expert_magic;              //   .
   string            m_expert_name;               //  .
   string            m_expert_symbol;             //    
   ENUM_TIMEFRAMES   m_timeframe;                 //  
   ENUM_TRADE_STATE  m_trade_state;               //   .
   CTradeState       m_state;                     //   .
   CTradeEnvironment m_environment;               //   .
   CArrayObj         m_bars_detecors;             //    .
   CArrayObj         m_ticks_detectors;           //    .

   void              RebuildPositions(void);
   void              CallInit(const MarketEvent &event);
   void              CallSupport(const MarketEvent &event);
   void              SpyEnvironment(void);
   virtual void      ExitByStopRegim(CPosition *pos);
   void              NewBarsDetect(void);
   void              NewTickDetect(void);
   void              InitSeries(string symbol,ENUM_TIMEFRAMES period);
   void              RecalcStatistic(PositionsStat &positions);
   int               LastWorkExpDay(MqlDateTime &dt);
protected:
   CArrayObj         ActivePositions;      //     .
   CArrayObj         ComplexPositions;     //         
   PositionsStat     positions;            //    
   CTradeControl     Trade;                //   ( CStrategy    ).
   CArrayObj         Modules;              //   
   static CLog*      Log;                  //  
/*      MQL4 */
   CTime             Time;
   COpen             Open;
   CHigh             High;
   CLow              Low;
   CClose            Close;
   CVolume           Volume;

   string            GetCurrentContract(string symbol);
   bool              CheckCurrentSL(double sl,ENUM_POSITION_TYPE type);
/*    "  "  "  " */
   bool              AddBarOpenEvent(string symbol,ENUM_TIMEFRAMES timeframe);
   bool              AddTickEvent(string symbol);
   void              CheckVolumes(void);
/*           ,
               
      */
   virtual void      OnSymbolChanged(string symbol);
   virtual void      OnTimeframeChanged(ENUM_TIMEFRAMES tf);
/*  ,    */
   virtual void      InitBuy(const MarketEvent &event);
   virtual void      InitSell(const MarketEvent &event);
   virtual void      InitComplexPos(const MarketEvent &event);
   virtual void      SupportBuy(const MarketEvent &event,CPosition *pos);
   virtual void      SupportSell(const MarketEvent &event,CPosition *pos);

public:
   CMoneyManagment   MM;          //   
                     CStrategy(void);
                     CStrategy(string symbol,string exp_name,uint magic,ENUM_TIMEFRAMES tf);
/* */
   static CStrategy *GetStrategy(string name);
   virtual bool      ParseXmlParams(CXmlElement *params);
   uint              ExpertMagic(void);
   void              ExpertMagic(uint ExpertMagic);
   string            ExpertName(void);
   void              ExpertName(string name);
   virtual string    ExpertNameFull(void);
   string            ExpertSymbol();
   void              ExpertSymbol(string symbol);
   void              Timeframe(ENUM_TIMEFRAMES period);
   ENUM_TIMEFRAMES   Timeframe(void);
   ENUM_TRADE_STATE  TradeState(void);
   void              TradeState(ENUM_TRADE_STATE state);

/*  */
   void              Buy(double vol);
   void              Sell(double vol);

/* */
   void              OnTick(void);
   void              OnTimer(void);
   void              OnBookEvent(string symbol);
   virtual void OnTradeTransaction(const MqlTradeTransaction &trans,
                                   const MqlTradeRequest &request,
                                   const MqlTradeResult &result){;}
                                   virtual int  Compare(const CObject *node,const int mode=0) const;
  };
/*  */
CLog             *CStrategy::Log;
//+------------------------------------------------------------------+
//|   -.                               |
//+------------------------------------------------------------------+
CStrategy::CStrategy(void)
  {
   m_last_changed = 0;
   m_expert_magic = 0;
   m_timeframe=PERIOD_CURRENT;
   Log=CLog::GetLog();
  }
//+------------------------------------------------------------------+
//|     .                       |
//+------------------------------------------------------------------+
CStrategy::CStrategy(string symbol,string exp_name,uint magic,ENUM_TIMEFRAMES tf)
  {
   m_last_changed=0;
   Log=CLog::GetLog();
   ExpertSymbol(symbol);
   ExpertName(exp_name);
   Timeframe(tf);
   ExpertMagic(magic);
  }
//+------------------------------------------------------------------+
//|    ,   |
//|          |
//|   ,            |
//|                                                  |
//+------------------------------------------------------------------+
void CStrategy::OnSymbolChanged(string symbol)
  {
  }
//+------------------------------------------------------------------+
//|    ,              |
//|          |
//|      ,        |
//|                               |
//+------------------------------------------------------------------+
void CStrategy::OnTimeframeChanged(ENUM_TIMEFRAMES tf)
  {
  }
//+------------------------------------------------------------------+
//|    ,    |
//|    .      |
//|      .                        |
//| IN:                                                              |
//|   event - ,  ,            |
//|               .                 |
//+------------------------------------------------------------------+
void CStrategy::InitBuy(const MarketEvent &event)
  {
  }
//+------------------------------------------------------------------+
//|    ,    |
//|    .     |
//|      .                        |
//| IN:                                                              |
//|   event - ,  ,            |
//|               .                 |
//+------------------------------------------------------------------+
void CStrategy::InitSell(const MarketEvent &event)
  {
  }
//+------------------------------------------------------------------+
//|    ,    |
//|    ( ) .        |
//|                       |
//|    .                                             |              
//| IN:                                                              |
//|   event - ,  ,            |
//|               .                 |
//+------------------------------------------------------------------+
void CStrategy::InitComplexPos(const MarketEvent &event)
  {
  }
//+------------------------------------------------------------------+
//|    ,    |
//|    ,    |
//| .            |
//|    .                                             |
//| IN:                                                              |
//|   event - ,  ,            |
//|               .                 |
//|   pos - ,    .        |
//+------------------------------------------------------------------+
void CStrategy::SupportBuy(const MarketEvent &event,CPosition *pos)
  {
  }
//+------------------------------------------------------------------+
//|    ,    |
//|    ,   |
//| .            |
//|    .                                             |
//| IN:                                                              |
//|   event - ,  ,            |
//|               .                 |
//|   pos - ,    .        |
//+------------------------------------------------------------------+
void CStrategy::SupportSell(const MarketEvent &event,CPosition *pos)
  {
  }
//+------------------------------------------------------------------+
//|                 |
//| ,    <Params>                       |
//+------------------------------------------------------------------+
bool CStrategy::ParseXmlParams(CXmlElement *xmlParams)
  {
   string text="Found specific xml-settings, but "+ExpertName()+" strategy does not handle them. Override the method ParseXmlParams";
   CMessage *msg=new CMessage(MESSAGE_WARNING,SOURCE,text);
   Log.AddMessage(msg);
   return false;
  }
//+------------------------------------------------------------------+
//|    .                  |
//+------------------------------------------------------------------+
void CStrategy::ExpertMagic(uint ExpertMagic)
  {
   m_expert_magic=ExpertMagic;
   Trade.SetExpertMagicNumber(m_expert_magic);
  }
//+------------------------------------------------------------------+
//|    .                    |
//+------------------------------------------------------------------+
uint CStrategy::ExpertMagic(void)
  {
   return m_expert_magic;
  }
//+------------------------------------------------------------------+
//|    (     |
//|       ).     |
//+------------------------------------------------------------------+
string CStrategy::ExpertName(void)
  {
   return m_expert_name;
  }
//+------------------------------------------------------------------+
//|        .           |
//+------------------------------------------------------------------+
void CStrategy::ExpertName(string name)
  {
   m_expert_name=name;
  }
//+------------------------------------------------------------------+
//|   (  ).                     |
//| (      )      |
//+------------------------------------------------------------------+
string CStrategy::ExpertNameFull(void)
  {
   return ExpertName();
  }
//+------------------------------------------------------------------+
//|    .                          |
//+------------------------------------------------------------------+
string CStrategy::ExpertSymbol(void)
  {
   if(m_expert_symbol==NULL || m_expert_symbol=="")
      return _Symbol;
   return m_expert_symbol;
  }
//+------------------------------------------------------------------+
//|    .                       |
//+------------------------------------------------------------------+
void CStrategy::ExpertSymbol(string symbol)
  {
   m_expert_symbol=GetCurrentContract(symbol);
   InitSeries(symbol,Timeframe());
   OnSymbolChanged(m_expert_symbol);
  }
//+------------------------------------------------------------------+
//|             |
//|  ' '.                                             |
//+------------------------------------------------------------------+
void CStrategy::OnTick(void)
  {
   NewTickDetect();
   NewBarsDetect();
  }
//+------------------------------------------------------------------+
//|             |
//|  'OnTimer'.                                               |
//+------------------------------------------------------------------+
void CStrategy::OnTimer(void)
  {
   m_event.symbol=Symbol();
   m_event.type=MARKET_EVENT_TIMER;
   m_event.period=(ENUM_TIMEFRAMES)Period();
   CallSupport(m_event);
   CallInit(m_event);
   NewTickDetect();
   NewBarsDetect();
  }
//+------------------------------------------------------------------+
//|             |
//|  'OnBookEvent'.                                           |
//+------------------------------------------------------------------+
void CStrategy::OnBookEvent(string symbol)
  {
   m_event.symbol=symbol;
   m_event.type=MARKET_EVENT_BOOK_EVENT;
   m_event.period=PERIOD_CURRENT;
   CallSupport(m_event);
   CallInit(m_event);
   NewTickDetect();
   NewBarsDetect();
  }
//+------------------------------------------------------------------+
//|      vol                             |
//+------------------------------------------------------------------+
void CStrategy::Buy(double vol)
  {
   Trade.Buy(vol,ExpertSymbol(),"hand buy");
  }
//+------------------------------------------------------------------+
//|      vol                              |
//+------------------------------------------------------------------+
void CStrategy::Sell(double vol)
  {
   Trade.Sell(vol,ExpertSymbol(),"hand sell");
  }
//+------------------------------------------------------------------+
//|     .                  |
//+------------------------------------------------------------------+
ENUM_TRADE_STATE CStrategy::TradeState(void)
  {
   return m_trade_state;
  }
//+------------------------------------------------------------------+
//|     .               |
//+------------------------------------------------------------------+
void CStrategy::TradeState(ENUM_TRADE_STATE state)
  {
   if(state!=m_trade_state)
     {
      m_trade_state=state;
      string text="The mode of the current strategy has been changed to "+EnumToString(m_trade_state)+
                  ". The changes will come into force at receipt of new events";
      CMessage *msg=new CMessage(MESSAGE_INFO,SOURCE,text);
      Log.AddMessage(msg);
     }
  }
//+------------------------------------------------------------------+
//|      ,         |
//|      .                        |
//+------------------------------------------------------------------+
void CStrategy::CallInit(const MarketEvent &event)
  {
   m_trade_state=m_state.GetTradeState();
   if(m_trade_state == TRADE_STOP)return;
   if(m_trade_state == TRADE_WAIT)return;
   if(m_trade_state == TRADE_NO_NEW_ENTRY)return;
   SpyEnvironment();
   InitComplexPos(event);
   if(m_trade_state==TRADE_BUY_AND_SELL || m_trade_state==TRADE_BUY_ONLY)
      InitBuy(event);
   if(m_trade_state==TRADE_BUY_AND_SELL || m_trade_state==TRADE_SELL_ONLY)
      InitSell(event);
  }
//+------------------------------------------------------------------+
//|      ,    |
//|    TRADE_WAIT.                                   |
//+------------------------------------------------------------------+
void CStrategy::CallSupport(const MarketEvent &event)
  {
   m_trade_state=m_state.GetTradeState();
   if(m_trade_state == TRADE_WAIT)return;
   SpyEnvironment();
   for(int i=ActivePositions.Total()-1; i>=0; i--)
     {
      CPosition *pos=ActivePositions.At(i);
      if(pos.ExpertMagic()!=m_expert_magic)continue;
      if(pos.Symbol()!=ExpertSymbol())continue;
      if(pos.Direction()==POSITION_TYPE_BUY)
         SupportBuy(event,pos);
      else
         SupportSell(event,pos);
      if(m_trade_state==TRADE_STOP && pos.IsActive())
         ExitByStopRegim(pos);
     }
  }
//+------------------------------------------------------------------+
//|    .                       |
//+------------------------------------------------------------------+
void CStrategy::SpyEnvironment(void)
  {
   if(m_environment.ChangeEnvironment())
     {
      printf(ExpertNameFull()+". Trading environment has changed. Rebuild the environment");
      RebuildPositions();
      RecalcStatistic(positions);
      m_environment.RememberEnvironment();
     }
  }
//+------------------------------------------------------------------+
//|           |
//| .                                                       |
//+------------------------------------------------------------------+
void CStrategy::RecalcStatistic(PositionsStat &pos)
  {
   pos.open_buy=0;
   pos.open_sell=0;
   pos.open_total=0;
   pos.open_complex=0;
   for(int i=0; i<ActivePositions.Total(); i++)
     {
      CPosition *cpos=ActivePositions.At(i);
      if(cpos.ExpertMagic()!=ExpertMagic())continue;
      pos.open_total+=1;
      if(cpos.Direction()==POSITION_TYPE_BUY)
         pos.open_buy++;
      else
         pos.open_sell++;
     }
  }
//+------------------------------------------------------------------+
//|   .                                    |
//+------------------------------------------------------------------+
void CStrategy::RebuildPositions(void)
  {
   ActivePositions.Clear();
   for(int i=0; i<PositionsTotal(); i++)
     {
      string symbol=PositionGetSymbol(i);
      PositionSelect(symbol);
      CPosition *pos=new CPosition();
      ActivePositions.Add(pos);
     }
  }
//+------------------------------------------------------------------+
//|    .                        |
//+------------------------------------------------------------------+
void CStrategy::Timeframe(ENUM_TIMEFRAMES period)
  {
   m_timeframe=period;
   InitSeries(ExpertSymbol(),m_timeframe);
   OnTimeframeChanged(m_timeframe);
  }
//+------------------------------------------------------------------+
//|    .                          |
//+------------------------------------------------------------------+
ENUM_TIMEFRAMES CStrategy::Timeframe(void)
  {
   return m_timeframe;
  }
//+------------------------------------------------------------------+
//|   OHLCV     . |
//+------------------------------------------------------------------+
void CStrategy::InitSeries(string symbol,ENUM_TIMEFRAMES period)
  {
   Time.Symbol(symbol);
   Time.Timeframe(period);
   Open.Symbol(symbol);
   Open.Timeframe(period);
   High.Symbol(symbol);
   High.Timeframe(period);
   Low.Symbol(symbol);
   Low.Timeframe(period);
   Close.Symbol(symbol);
   Close.Timeframe(period);
   Volume.Symbol(symbol);
   Volume.Timeframe(period);
  }
//+------------------------------------------------------------------+
//|     Stop.                                |
//+------------------------------------------------------------------+
void CStrategy::ExitByStopRegim(CPosition *pos)
  {
   ResetLastError();
   CMessage *msg_info=new CMessage(MESSAGE_INFO,ExpertName(),"Try close position #"+(string)pos.ID()+" by stop regim...");
   Log.AddMessage(msg_info);
   bool res=pos.CloseAtMarket("Exit by StopRegim");
   if(res)
      CMessage *msg=new CMessage(MESSAGE_INFO,ExpertName(),"Out of position #"+(string)pos.ID()+" successfully completed.");
//    ,       pos.CloseAtMarket.
  }
//+------------------------------------------------------------------+
//|         |
//|   .                                            |
//+------------------------------------------------------------------+
void CStrategy::NewBarsDetect(void)
  {
   if(m_bars_detecors.Total()==0)
      AddBarOpenEvent(ExpertSymbol(),Timeframe());
   for(int i=0; i<m_bars_detecors.Total(); i++)
     {
      CBarDetector *bar=m_bars_detecors.At(i);
      if(bar.IsNewBar())
        {
         m_event.period = bar.Timeframe();
         m_event.symbol = bar.Symbol();
         m_event.type=MARKET_EVENT_BAR_OPEN;
         CallSupport(m_event);
         CallInit(m_event);
        }
     }
  }
//+------------------------------------------------------------------+
//|      .         |
//+------------------------------------------------------------------+
void CStrategy::NewTickDetect(void)
  {
   if(m_ticks_detectors.Total()==0)
      AddTickEvent(ExpertSymbol());
   for(int i=0; i<m_ticks_detectors.Total(); i++)
     {
      CTickDetector *tick=m_ticks_detectors.At(i);
      if(tick.IsNewTick())
        {
         m_event.period=PERIOD_CURRENT;
         m_event.type=MARKET_EVENT_TICK;
         m_event.symbol=tick.Symbol();
         CallSupport(m_event);
         CallInit(m_event);
        }
     }
  }
//+------------------------------------------------------------------+
//|           |
//|   symbol   timframe.       |
//|    ,   .       |
//|  ,  .                             |
//+------------------------------------------------------------------+
bool CStrategy::AddBarOpenEvent(string symbol,ENUM_TIMEFRAMES timeframe)
  {
   for(int i=0; i<m_bars_detecors.Total(); i++)
     {
      CBarDetector *d=m_bars_detecors.At(i);
      if(d.Symbol()==symbol && d.Timeframe()==timeframe)
        {
         string text="You are already subscribed to the opening bars of said symbol and timeframe.";
         CMessage *message=new CMessage(MESSAGE_INFO,__FUNCTION__,text);
         return false;
        }
     }
   datetime time[];
   if(CopyTime(symbol,timeframe,0,3,time)==0)
     {
      string text="A symbol "+symbol+" that you want to monitor is not available in the terminal."+
                  " Make sure that the name of the instrument and its timeframe"+EnumToString(timeframe)+" are correct.";
      CMessage *message=new CMessage(MESSAGE_WARNING,__FUNCTION__,text);
      return false;
     }
   CBarDetector *bar=new CBarDetector(symbol,timeframe);
   return m_bars_detecors.Add(bar);
  }
//+------------------------------------------------------------------+
//|      " ",           |
//|   symbol.             |
//| ,   .                          |
//|  ,  .                             |
//+------------------------------------------------------------------+
bool CStrategy::AddTickEvent(string symbol)
  {
   for(int i=0; i<m_bars_detecors.Total(); i++)
     {
      CTickDetector *d=m_ticks_detectors.At(i);
      if(d.Symbol()==symbol)
        {
         string text="You are already subscribed to new tick event of said symbol.";
         CMessage *message=new CMessage(MESSAGE_WARNING,__FUNCTION__,text);
         return false;
        }
     }
   datetime time[];
   if(CopyTime(symbol,PERIOD_D1,0,3,time)==0)
     {
      string text="A symbol "+symbol+" that you want to monitor is not available in the terminal."+
                  " Make sure that the name of the instrument are correct.";
      CMessage *message=new CMessage(MESSAGE_WARNING,__FUNCTION__,text);
      return false;
     }
   CTickDetector *tick=new CTickDetector(symbol);
   return m_ticks_detectors.Add(tick);
  }
//+------------------------------------------------------------------+
//|       FORTS,       |
//|   . ,      |
//|  02.10.2015,     "Si",     |
//|   : Si-12.15.                         |
//|            |
//|   ,    ,      |
//| NULL.       . |
//+------------------------------------------------------------------+
string CStrategy::GetCurrentContract(string symbol)
  {
   datetime time[];
   if(SymbolInfoInteger(symbol,SYMBOL_SELECT))
      return symbol;
   MqlDateTime dt={0};
   TimeCurrent(dt);
   int mon=0;
   int year=0;
   if((dt.mon==12 && dt.day>=14) || dt.mon==1 || dt.mon==2 || (dt.mon==3 && dt.day<14))
     {
      mon=3;
      year=dt.year+1;
     }
   if((dt.mon==3 && dt.day>=14) || dt.mon==4 || dt.mon==5 || (dt.mon==6 && dt.day<14))
     {
      mon=6;
      year=dt.year;
     }
   if((dt.mon==6 && dt.day>=14) || dt.mon==7 || dt.mon==8 || (dt.mon==9 && dt.day<14))
     {
      mon=9;
      year=dt.year;
     }
   if((dt.mon==9 && dt.day>=14) || dt.mon==10 || dt.mon==11 || (dt.mon==12 && dt.day<14))
     {
      mon=12;
      year=dt.year;
     }
   if(mon==0 || year==0)
     {
      string text="The current date is outside the execution of futures";
      CMessage *msg=new CMessage(MESSAGE_ERROR,__FUNCTION__,text);
      return NULL;
     }
//       
   string full_symbol=symbol+"-"+(string)mon+"."+StringSubstr((string)year,2);
   if(!SymbolInfoInteger(full_symbol,SYMBOL_SELECT))
     {
      string text="Symbol "+symbol+" is not not selected in market watch. Check the name of the instrument";
      CMessage *msg=new CMessage(MESSAGE_ERROR,__FUNCTION__,text);
      Log.AddMessage(msg);
      return NULL;
     }
   return full_symbol;
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
int CStrategy::Compare(const CObject *obj,const int mode=0)const
  {
   const CStrategy *str=obj;
   if(m_expert_magic > str.m_expert_magic)return 1;
   if(m_expert_magic < str.m_expert_magic)return -1;
   return 0;
  }
#include <Strategy\StrategyFactory.mqh>
//+------------------------------------------------------------------+
